/*-
 * Copyright (c) 2012,2013 Darran Hunt (darran [at] hunt dot net dot nz)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @file WiFly RN-XV Library
 */



//############################################################################
//# HISTORY  PKT_WiFlyHQ.c
//#
//# noch UNVOLLSTNDIG!! (Cebra)
//#
//# 12.02.2014 OG
//# - chg: readTimeout() Auskommentiert: "unused variable 'ind'"
//# - chg: wl_init() Auskommentiert: "unused variable 'dhcpMode'"
//#
//# 02.07.2013 cebra
//# - new: Routinen fr WiFly WLAN-Modul
//#
//###########################################################################


#include <avr/pgmspace.h>
#include <stdbool.h>
#include <stdlib.h>
#include <stdarg.h>
#include <util/delay.h>
#include <inttypes.h>
#include <string.h>
#include "../main.h"
#include "PKT_WiFlyHQ.h"
#include "../eeprom/eeprom.h"
#include "../uart/uart1.h"
#include "../timer/timer.h"
#include "../lcd/lcd.h"

#ifdef USE_WLAN

/* Change these to match your WiFi network */
const char mySSID[] = "PKT";
const char myPassword[] = "123456";




/* For free memory check */
//extern unsigned int __bss_end;
//extern unsigned int __heap_start;
//extern void *__brkval;

#undef DEBUG

#ifdef DEBUG
#define DPRINT(item) debug.print(item)
#define DPRINTLN(item) debug.println(item)
#else
#define DPRINT(item)
#define DPRINTLN(item)
#endif

#define WIFLY_STATUS_TCP_MASK 		0x000F
#define WIFLY_STATUS_TCP_OFFSET		0
#define WIFLY_STATUS_ASSOC_MASK 	0x0001
#define WIFLY_STATUS_ASSOC_OFFSET	4
#define WIFLY_STATUS_AUTHEN_MASK 	0x0001
#define WIFLY_STATUS_AUTHEN_OFFSET	5
#define WIFLY_STATUS_DNS_SERVER_MASK	0x0001
#define WIFLY_STATUS_DNS_SERVER_OFFSET	6
#define WIFLY_STATUS_DNS_FOUND_MASK	0x0001
#define WIFLY_STATUS_DNS_FOUND_OFFSET	7
#define WIFLY_STATUS_CHAN_MASK 		0x000F
#define WIFLY_STATUS_CHAN_OFFSET	9

#define WIFLY_TCP_IDLE		0
#define WIFLY_TCP_CONNECTED	1
#define WIFLY_TCP_NOIP		3
#define WIFLY_TCP_CONNECTING	4

/* WiFi data rates */
#define WIFLY_RATE_1MBPS	0
#define WIFLY_RATE_2MBPS	1
#define WIFLY_RATE_5_5MBPS	2	/* 5.5 MBps */
#define WIFLY_RATE_6MBPS	8
#define WIFLY_RATE_9MBPS	9
#define WIFLY_RATE_11MBPS	3
#define WIFLY_RATE_12MBPS	10
#define WIFLY_RATE_18MBPS	11
#define WIFLY_RATE_24MBPS	12	/* Default */
#define WIFLY_RATE_36MBPS	13
#define WIFLY_RATE_48MBPS	14
#define WIFLY_RATE_54MBPS	15

/* Work around a bug with PROGMEM and PSTR where the compiler always
 * generates warnings.
 */
//#undef PROGMEM
//#define PROGMEM __attribute__(( section(".progmem.data") ))
//#undef PSTR
//#define PSTR(s) (__extension__({static char __c[] PROGMEM = (s); &__c[0];}))

/* Request and response strings in PROGMEM */
const char req_GetIP[] PROGMEM = "get ip\r";
const char resp_IP[] PROGMEM = "IP=";
const char resp_NM[] PROGMEM = "NM=";
const char resp_GW[] PROGMEM = "GW=";
const char resp_Host[] PROGMEM = "HOST=";
const char resp_DHCP[] PROGMEM = "DHCP=";
const char req_GetMAC[] PROGMEM = "get mac\r";
const char resp_MAC[] PROGMEM = "Mac Addr=";
const char req_GetWLAN[] PROGMEM = "get wlan\r";
const char resp_SSID[] PROGMEM = "SSID=";
const char resp_Chan[] PROGMEM = "Chan=";
const char req_GetOpt[] PROGMEM = "get opt\r";
const char resp_DeviceID[] PROGMEM = "DeviceId=";
const char req_GetUart[] PROGMEM = "get u\r";
const char resp_Baud[] PROGMEM = "Baudrate=";
const char req_GetTime[] PROGMEM = "get time\r";
const char resp_Zone[] PROGMEM = "Zone=";
const char req_ShowTime[] PROGMEM = "show time\r";
const char resp_Uptime[] PROGMEM = "UpTime=";
const char resp_Time[] PROGMEM = "Time=";
const char req_GetDNS[] PROGMEM = "get dns\r";
const char resp_DNSAddr[] PROGMEM = "Address=";
const char req_ShowTimeT[] PROGMEM = "show t t\r";
const char resp_RTC[] PROGMEM = "RTC=";
const char resp_Mode[] PROGMEM = "Mode=";
const char req_GetComm[] PROGMEM = "get comm\r";
const char resp_FlushTimeout[] PROGMEM = "FlushTimer=";
const char resp_FlushChar[] PROGMEM = "MatchChar=";
const char resp_FlushSize[] PROGMEM = "FlushSize=";
const char req_GetRSSI[] PROGMEM = "show rssi\r";
const char resp_RSSI[] PROGMEM = "RSSI=(-";
const char resp_Flags[] PROGMEM = "FLAGS=0x";
const char resp_Protocol[] PROGMEM = "PROTO=";
const char req_GetAdhoc[] PROGMEM = "get adhoc\r";
const char resp_Beacon[] PROGMEM = "Beacon=";
const char resp_Probe[] PROGMEM = "Probe=";
const char resp_Reboot[] PROGMEM = "Reboot=";
const char resp_Join[] PROGMEM = "Join=";
const char resp_Rate[] PROGMEM = "Rate=";
const char resp_Power[] PROGMEM = "TxPower=";
const char resp_Replace[] PROGMEM = "Replace=";
const char req_GetVersion[] PROGMEM = "ver\r";
const char resp_Version[] PROGMEM = "WiFly";

/* Request and response for specific info */
static struct {
    const char *req;
    const char *resp;
} requests[] = {
    { req_GetIP,        resp_IP },       /* 0 */
    { req_GetIP,        resp_NM },       /* 1 */
    { req_GetIP,        resp_GW },       /* 2 */
    { req_GetMAC,       resp_MAC },      /* 3 */
    { req_GetWLAN,      resp_SSID },     /* 4 */
    { req_GetOpt,       resp_DeviceID }, /* 5 */
    { req_GetUart,      resp_Baud },     /* 6 */
    { req_ShowTime,     resp_Time },     /* 7 */
    { req_ShowTime,     resp_Uptime },   /* 8 */
    { req_GetTime,      resp_Zone },     /* 9 */
    { req_GetDNS,       resp_DNSAddr },  /* 10 */
    { req_ShowTimeT,    resp_RTC },      /* 11 */
    { req_GetIP,        resp_DHCP },     /* 12 */
    { req_GetUart,      resp_Mode },     /* 13 */
    { req_GetComm,      resp_FlushTimeout }, /* 14 */
    { req_GetComm,      resp_FlushChar }, /* 15 */
    { req_GetComm,      resp_FlushSize }, /* 16 */
    { req_GetRSSI,      resp_RSSI },     /* 17 */
    { req_GetIP,        resp_Flags },    /* 18 */
    { req_GetIP,        resp_Host },     /* 19 */
    { req_GetIP,        resp_Protocol }, /* 20 */
    { req_GetAdhoc,     resp_Beacon },   /* 21 */
    { req_GetAdhoc,     resp_Probe },    /* 22 */
    { req_GetAdhoc,     resp_Reboot },   /* 23 */
    { req_GetWLAN,      resp_Join },     /* 24 */
    { req_GetWLAN,      resp_Rate },     /* 25 */
    { req_GetWLAN,      resp_Power },    /* 26 */
    { req_GetOpt,       resp_Replace },  /* 27 */
    { req_GetVersion,   resp_Version },  /* 28 */
};

/* Request indices, must match table above */
typedef enum {
    WIFLY_GET_IP        = 0,
    WIFLY_GET_NETMASK   = 1,
    WIFLY_GET_GATEWAY   = 2,
    WIFLY_GET_MAC       = 3,
    WIFLY_GET_SSID      = 4,
    WIFLY_GET_DEVICEID  = 5,
    WIFLY_GET_BAUD      = 6,
    WIFLY_GET_TIME      = 7,
    WIFLY_GET_UPTIME    = 8,
    WIFLY_GET_ZONE      = 9,
    WIFLY_GET_DNS       = 10,
    WIFLY_GET_RTC       = 11,
    WIFLY_GET_DHCP      = 12,
    WIFLY_GET_UART_MODE = 13,
    WIFLY_GET_FLUSHTIMEOUT = 14,
    WIFLY_GET_FLUSHCHAR = 15,
    WIFLY_GET_FLUSHSIZE = 16,
    WIFLY_GET_RSSI      = 17,
    WIFLY_GET_IP_FLAGS  = 18,
    WIFLY_GET_HOST      = 19,
    WIFLY_GET_PROTOCOL  = 20,
    WIFLY_GET_BEACON    = 21,
    WIFLY_GET_PROBE     = 22,
    WIFLY_GET_REBOOT    = 23,
    WIFLY_GET_JOIN      = 24,
    WIFLY_GET_RATE      = 25,
    WIFLY_GET_POWER     = 26,
    WIFLY_GET_REPLACE   = 27,
    WIFLY_GET_VERSION   = 28,
} e_wifly_requests;



bool inCommandMode;
int  exitCommand;
bool dhcp;
bool restoreHost;
bool restoreHostStored;
char lastHost[32];
uint16_t lastPort;

bool tcpMode;
bool udpAutoPair;

bool tcpMode;
bool udpAutoPair;

bool connected;
bool connecting;

bool debugOn;

char buf[80];

uint8_t Result;
uint8_t Old_Baudrate;

/*  for dbgDump() */
char *dbgBuf;
int dbgInd;
int dbgMax;

char replaceChar;      /* The space replacement character */
static char prompt[16];
static bool gotPrompt = false;
/**
 * Convert a unsigned int to a string
 * @param val the value to convert to a string
 * @param base format for string; either DEC for decimal or
 *             HEX for hexidecimal
 * @param buf the buffer to write the string to
 * @param size the size of the buffer
 * @returns number of characters written to the buffer
 *          not including the null terminator (i.e. size of the string)
 **/



int simple_utoa(uint32_t val, uint8_t base, char *buf, int size)
{
    char tmpbuf[16];
    int ind=0;
    uint32_t nval;
    int fsize=0;

    if (base == DEC) {
        do {
            nval = val / 10;
            tmpbuf[ind++] = '0' + val - (nval * 10);
            val = nval;
        } while (val);
    } else {
        do {
            nval = val & 0x0F;
            tmpbuf[ind++] = nval + ((nval < 10) ? '0' : 'A');
            val >>= 4;
        } while (val);
        tmpbuf[ind++] = 'x';
        tmpbuf[ind++] = '0';
    }

    ind--;

    do {
        buf[fsize++] = tmpbuf[ind];
    } while ((ind-- > 0) && (fsize < (size-1)));
    buf[fsize] = '\0';

    return fsize;
}

/** Simple hex string to uint32_t */
uint32_t atoh(char *buf)
{
    uint32_t res=0;
    char ch;
    bool gotX = false;

    while ((ch=*buf++) != 0) {
        if (ch >= '0' && ch <= '9') {
            res = (res << 4) + ch - '0';
        } else if (ch >= 'a' && ch <= 'f') {
            res = (res << 4) + ch - 'a' + 10;
        } else if (ch >= 'A' && ch <= 'F') {
            res = (res << 4) + ch - 'A' + 10;
        } else if ((ch == 'x') && !gotX) {
            /* Ignore 0x at start */
            gotX = true;
        } else {
            break;
        }
    }

    return res;
}

/** Simple ASCII to unsigned int */
uint32_t atou(const char *buf)
{
    uint32_t res=0;

    while (*buf) {
        if ((*buf < '0') || (*buf > '9')) {
            break;
        }
        res = res * 10 + *buf - '0';
        buf++;
    }

    return res;
}



//}

/** Send a string to the WiFly */
void sends(const char *str)
{
//    DPRINT(F("send: ")); DPRINT(str); DPRINT("\n\r");
//    print(str);
    uart1_puts(str);
    //serial->print(str);
}

/** Send a character to the WiFly */
void sendc(const char ch)
{
//    write(ch);
    //serial->write(ch);
    uart1_putc(ch);
}


/** Send a string from PROGMEM to the WiFly */
void send_P(const char *str)
{
//    DPRINT(F("send_P: "));
//    DPRINTLN((const __FlashStringHelper *)str);

//    print((const __FlashStringHelper *)str);
    uart1_puts_p(str);
}

/**
 * Flush the incoming data from the WiFly.
 * @param timeout - the number of milliseconds to wait for additional data to flush. Default is 500msecs.
 */
void flushRx(int timeout)
{
    char ch;
//    DPRINT(F("flush\n\r"));
    while (readTimeout(&ch,timeout));
//    DPRINT(F("flushed\n\r"));
}



/**
 * Write a byte to the WiFly.
 * @param byte - the byte to write.
 * @return the number of bytes written (1).
 */
uint8_t write(uint8_t byte)
{
    if (dbgInd < dbgMax) {
        dbgBuf[dbgInd++] = byte;
    }
    uart1_putc(byte);
    // serial->write(byte);
    return 1;
}


/** Read the next character from the WiFly serial interface.
 * Waits up to timeout milliseconds to receive the character.
 * @param chp pointer to store the read character in
 * @param timeout the number of milliseconds to wait for a character
 * @retval true - character read
 * @retval false - timeout reached, character not read
 */
bool readTimeout(char *chp, uint16_t timeout)
{
//    uint32_t start = millis();
    char ch;
    timer2 = timeout;
    //static int ind=0;

//    while (millis() - start < timeout)
      while (timer2>0)
      {
        if (uart1_available() > 0) {
            ch = uart1_getc();
//            lcd_print_hex(ch,0);
            *chp = ch;
//            if (dbgInd < dbgMax) {
//                dbgBuf[dbgInd++] = ch;
//            }
//            if (debugOn) {
//                debug.print(ind++);
//                debug.print(F(": "));
//                debug.print(ch,HEX);
//                if (isprint(ch)) {
//                    debug.print(' ');
//                    debug.print(ch);
//                }
//                debug.println();

//            }
            return true;
        }
    }

//    if (debugOn) {
//        debug.println(F("readTimeout - timed out"));
//    }

    return false;
}



/** Scan the input data for the WiFLy prompt.  This is a string starting with a '<' and
 * ending with a '>'. Store the prompt for future use.
 */
bool setPrompt()
{
    char ch;

    while (readTimeout(&ch,WIFLY_DEFAULT_TIMEOUT)) {
        if (ch == '<') {
            uint8_t ind = 1;
            prompt[0] = ch;
            while (ind < (sizeof(prompt)-4)) {
                if (readTimeout(&ch,WIFLY_DEFAULT_TIMEOUT)) {
                    prompt[ind++] = ch;
                    if (ch == '>') {
                        if (readTimeout(&ch,WIFLY_DEFAULT_TIMEOUT)) {
                            if (ch == ' ') {
                                prompt[ind++] = ch;
                                //prompt[ind++] = '\r';
                                //prompt[ind++] = '\n';
                                prompt[ind] = 0;
//                                DPRINT(F("setPrompt: ")); DPRINT(prompt); DPRINT("\n\r");
                                gotPrompt = true;
                                gets(NULL,0,WIFLY_DEFAULT_TIMEOUT);
                                return true;
                            } else {
                                /* wrong character */
                                return false;
                            }
                        } else {
                            /* timeout */
                            return false;
                        }
                    }
                } else {
                    /* timeout */
                    return false;
                }
            }

            return false;
        }
    }

    return false;
}


/**
 * Read characters from the WiFly until the prompt string is seen.
 * Characters are discarded until the prompt is found.
 * Fails if no data is received for the timeout duration.
 * @timeout fail if no data received for this period (in milliseconds).
 * @retval true - prompt found
 * @retval false - prompt not found, timeout reached
 * @Note: The first time this function is called it will set the prompt string.
 */
bool getPrompt(uint16_t timeout)
{
    bool res;

    if (!gotPrompt) {
//        DPRINT(F("setPrompt\n\r"));

        res = setPrompt();
        if (!res) {
//            debug.println(F("setPrompt failed"));
        }
    } else {
//        DPRINT(F("getPrompt \"")); DPRINT(prompt); DPRINT("\"\n\r");
        res = match(prompt, timeout);
    }
    return res;
}


/**
 * Read characters from the WiFly and match them against the
 * progmem string. Ignore any leading characters that don't match. Keep
 * reading, discarding the input, until the string is matched
 * or until no characters are received for the timeout duration.
 * @param str The string to match, in progmem.
 * @param timeout fail if no data received for this period (in milliseconds).
 * @retval true - a match was found
 * @retval false - no match found, timeout reached
 */
bool match_P(const char *str, uint16_t timeout)
{
    const char *match = str;
    char ch, ch_P;

//    if (debugOn) {
//        debug.print(F("match_P: "));
//        debug.println((const __FlashStringHelper *)str);
//        lcd_printp_at( 0, 0, PSTR("match_P: "), MNORMAL);
//        lcd_printp_at( 10, 0, str, MNORMAL);
//    }

    ch_P = pgm_read_byte(match);
    if (ch_P == '\0') {
        /* Null string always matches */
        return true;
    }

    while (readTimeout(&ch,timeout)) {
        if (ch == ch_P) {
            match++;
        } else {
            /* Restart match */
            match = str;
            if (ch == pgm_read_byte(match)) {
                match++;
            }
        }

        ch_P = pgm_read_byte(match);
        if (ch_P == '\0') {
//            DPRINT(F("match_P: true\n\r"));
//            lcd_printp_at( 0, 1, PSTR("match_P: true  "), MNORMAL);
            return true;
        }
    }

//    DPRINT(F("match_P: false\n\r"));
//    lcd_printp_at( 0, 1, PSTR("match_P: false"), MNORMAL);
    return false;
}



/** Put the WiFly into command mode */
bool enterCommandMode(void)
{
    uint8_t retry;

    if (inCommandMode) {
        return true;
    }

    _delay_ms(250);
    send_P(PSTR("$$$"));
    _delay_ms(250);
    if (match_P(PSTR("CMD\r\n"), WIFLY_COMMAND_TIMEOUT)) {
        /* Get the prompt */
        if (gotPrompt) {
            inCommandMode = true;
            return true;
        } else {
            for (retry=0; retry < 2; retry++) {
//                serial->write('\r');
                uart1_putc('\r');

                if (getPrompt(WIFLY_COMMAND_TIMEOUT)) {
                    inCommandMode = true;
                    return true;
                }
            }
        }
    }
    /* See if we're already in command mode */
//    DPRINT(F("Check in command mode\n\r"));
//    serial->write('\r');
    uart1_putc('\r');
    if (getPrompt(WIFLY_COMMAND_TIMEOUT)) {
        inCommandMode = true;
//        DPRINT(F("Already in command mode\n\r"));
        return true;
    }

    for (retry=0; retry<2; retry++) {
//        DPRINT(F("send $$$ ")); DPRINT(retry); DPRINT("\n\r");
        _delay_ms(250);
        send_P(PSTR("$$$"));
        _delay_ms(250);
        if (match_P(PSTR("CMD\r\n"), WIFLY_COMMAND_TIMEOUT)) {
            inCommandMode = true;
            return true;
        }
    }

    return false;
}



/* Get the WiFly ready to receive a command. */
bool startCommand(void)
{
    if (!inCommandMode) {
        if (!enterCommandMode()) {
            return false;
        }
        /* If we're already in command mode, then we don't exit it in finishCommand().
         * This is an optimisation to avoid switching in and out of command mode
         * when using several commands to implement another command.
         */
    } else {
//        DPRINT(F("Already in command mode\n\r"));
    }
    exitCommand++;
    return true;
}


/**
* Get WiFly ready to handle commands, and determine
* some initial status.
*/
uint8_t wl_init(void)
{
//int8_t dhcpMode=0;
uint8_t result = true;

inCommandMode = false;
exitCommand = 0;
connected = false;
connecting = false;
dhcp = true;
restoreHost = true;
#ifdef DEBUG
debugOn = true;
#else
debugOn = false;
#endif

dbgBuf = NULL;
dbgInd = 0;
dbgMax = 0;
lastPort = 0;
lastHost[0] = 0;


set_Modul_On(Wlan);
_delay_ms(4000);

if (!find_WiFly()) {
    lcd_printp_at( 0, 2, PSTR("WiFly not found"), MNORMAL);
    return false;
   }



//
//lcd_printp_at( 0, 4, PSTR("Wlan setbaud"), MNORMAL);
//
//if (!setBaud(Config.PKT_Baudrate))
//  {
//  lcd_printp_at( 0, 4, PSTR("Fail set Baud"), MNORMAL);
//  Result = false;
//  }

if(!setBaud(Config.PKT_Baudrate)){

    lcd_printp_at( 0, 2, PSTR("Error SetBaudrate"), MNORMAL);
    SetBaudUart1(Config.PKT_Baudrate);
    result = false;
   }

SetBaudUart1(Config.PKT_Baudrate);




if (result)
  {
    if (!setopt_RM(PSTR("set u m 1"), (char *)NULL, (char *)NULL,false)) {
        lcd_printp_at( 0, 3, PSTR("F turn off echo"), MNORMAL);
        result = false;
    }

//    if (!setopt_RM(PSTR("set sys printlvl 0"), (char *)NULL, (char *)NULL,false)) {
//    //    debug.println(F("Failed to turn off sys print"));
//        lcd_printp_at( 0, 4, PSTR("F t off sys pr"), MNORMAL);
//        result = false;
//    }
    if (!setopt_RM(PSTR("set comm remote 0"), (char *)NULL, (char *)NULL,false)) {
        lcd_printp_at( 0, 5, PSTR("F set comm rem"), MNORMAL);
        result = false;
    }

    lcd_puts_at( 0, 6, getVersion(buf, sizeof(buf)), MNORMAL);

    if (!Config.Wlan_WPA)
      {
       lcd_printp_at( 0, 4, PSTR("Init Adhoc Network"), MNORMAL);
       if (createAdhocNetwork(Config.Wlan_SSID, Config.Wlan_Channel)>0){
         lcd_printp_at( 0, 4, PSTR("Fail Init Adhoc "), MNORMAL);
         lcd_print_hex(Result,0);

         result = false;
       }

      }

    else
      {
//
//
//
//    // Adhoc = 4, WPA =2
//    if (!Config.Wlan_WPA) {
//      if (!setopt_RM(PSTR("set wlan join 4"), (char *)NULL, (char *)NULL,false)) {
//              lcd_printp_at( 0, 5, PSTR("F set Wlan Join"), MNORMAL);
//              result = false;
//      }
//    }
//    else
//      if (!setopt_RM(PSTR("set wlan join 2"), (char *)NULL, (char *)NULL,false)) {
//                    lcd_printp_at( 0, 5, PSTR("F set Wlan Join"), MNORMAL);
//                    result = false;
//            }
//
//    // Wlan Channel
//    if (!setopt_RM(PSTR("set wlan channel 1"), (char *)NULL, (char *)NULL,false)) {
//            lcd_printp_at( 0, 5, PSTR("F set Wlan Chan"), MNORMAL);
//            result = false;
//    }
//    // SSID
//    if (!setSSID(Config.Wlan_SSID)) {
//            lcd_printp_at( 0, 5, PSTR("F set SSID"), MNORMAL);
//            result = false;
//    }
//    // Password
//    if (Config.Wlan_WPA) {
//        if (!setPassphrase(Config.Wlan_Password)) {
//                lcd_printp_at( 0, 5, PSTR("F set Password"), MNORMAL);
//                result = false;
//        }
//
//
//    }
//    if (!setDHCP(Config.Wlan_DHCP)){
//                   lcd_printp_at( 0, 5, PSTR("F set DHCP"), MNORMAL);
//                   result = false;
//          }
//
//    if (!setIP(Config.Wlan_IP)) {
//            lcd_printp_at( 0, 5, PSTR("F set IP"), MNORMAL);
//            result = false;
//    }
//
//    if (!setNetmask(Config.Wlan_Netmask)) {
//            lcd_printp_at( 0, 5, PSTR("F set Mask"), MNORMAL);
//            result = false;
//    }
//
//    if (!setGateway(Config.Wlan_Gateway)) {
//            lcd_printp_at( 0, 5, PSTR("F set GW"), MNORMAL);
//            result = false;
//    }
//    if (!exitCommandMode()) {
//        lcd_printp_at( 0, 6, PSTR("F exit com mode"), MNORMAL);
//         result = false;
//    }
//
//    if (!save()) {
//        result = false;
//        lcd_printp_at( 0, 7, PSTR("save Error"), MNORMAL);
//    }

  }

//    while(!get_key_press (1 << KEY_ESC));

  }



/* update connection status */
//getConnection();
//
//DPRINT(F("tcp status: ")); DPRINT(status.tcp); DPRINT("\n\r");
//DPRINT(F("assoc status: ")); DPRINT(status.assoc); DPRINT("\n\r");
//DPRINT(F("authen status: ")); DPRINT(status.authen); DPRINT("\n\r");
//DPRINT(F("dns status: ")); DPRINT(status.dnsServer); DPRINT("\n\r");
//DPRINT(F("dns found status: ")); DPRINT(status.dnsFound); DPRINT("\n\r");
//DPRINT(F("channel status: ")); DPRINT(status.channel); DPRINT("\n\r");
//
//dhcpMode = getDHCPMode();
//dhcp = !((dhcpMode == WIFLY_DHCP_MODE_OFF) || (dhcpMode == WIFLY_DHCP_MODE_SERVER));
//
//replaceChar = getSpaceReplace();
set_Modul_On(USB);
if (!result) _delay_ms(4000);
return result;

}

/**
 * Set an option to an unsigned integer value
 * @param opt the set command to use (in Flash)
 * @param value the value to set
 * @param base the number format; DEC or HEX
 * @returns true on success, false on failure
 */
bool setopt(const char *opt, const uint32_t value, uint8_t base)
{
    char buf[11];
    simple_utoa(value, base, buf, sizeof(buf));
    return setopt_RM(opt, buf,(char *)NULL,false);
}

/**
 * Set an option, confirm ok status
 * The value to set can either be in RAM or in Flash.
 * @param cmd the set command to use (string in Flash)
 * @param buf the value to set the option to (string in RAM). Set to NULL if not used.
 * @param buf_p the value to set the option to (string in Flash). Set to NULL if not used.
 * @param spacesub set to true to have spaces replaced with the
 *        current replacement character
 * @returns true on success, false on failure
 */

//(!setopt_RM(PSTR("set u m 1"), (char *)NULL, (char *)NULL,false))

uint8_t setopt_RM(const char *cmd, const char *buf, const char *buf_P, uint8_t spacesub)
{
    char rbuf[16];
    bool res;

    if (!startCommand()) {
        return false;
    }

    send_P(cmd);
    if (buf_P != NULL) {
        sendc(' ');
        if (spacesub) {
            const char *str = (const char *)buf_P;
            char ch;
            while ((ch = pgm_read_byte(str++)) != 0) {
                if (ch == ' ') {
                    write(replaceChar);
                } else {
                    write(ch);
                }
            }
        } else {
            send_P((const char *)buf_P);
        }
    } else if (buf != NULL) {
        sendc(' ');
        if (spacesub) {
            const char *str = buf;
            char ch;
            /* spaces must be replaced */
            while ((ch = *str++) != 0) {
                if (ch == ' ') {
                    write(replaceChar);
                } else {
                    write(ch);
                }
            }
        } else {
            sends(buf);
        }
    }
    sendc('\r');

    res = getres(rbuf, sizeof(rbuf));
    getPrompt(0);

    finishCommand();
    return res;
}
/**
 * Set an option to an unsigned integer value
 * @param opt the set command to use (in Flash)
 * @param value the value to set
 * @param base the number format; DEC or HEX
 * @returns true on success, false on failure
 */
uint8_t setopt_p(const char *opt, const uint32_t value, uint8_t base)
{
    char buf[11];
    simple_utoa(value, base, buf, sizeof(buf));
    return setopt_RM(opt, buf,NULL,false);
}



/* Save current configuration */
bool save(void)
{
    bool res = false;

    if (!startCommand()) {
        return false;
    }
    send_P(PSTR("save\r"));
    if (match_P(PSTR("Storing"),WIFLY_DEFAULT_TIMEOUT)) {
        getPrompt(WIFLY_DEFAULT_TIMEOUT);
        res = true;
    }

    finishCommand();
    return res;
}










/**
 * Read characters into the buffer until a carriage-return and newline is reached.
 * If the buffer is too small, the remaining characters in the line are discarded.
 * @param buf - the buffer to read into. If this is NULL then all characters in the line are discarded.
 * @param size - the size of the buffer (max number of characters it can store)
 * @param timeout - the number of milliseconds to wait for a new character to arrive
 * @returns the number of characters read into the buffer.
 * @retval 0 - timeout reading newline
 * @note The buffer will be null terminated, and in effect can hold size-1 characters.
 */
int gets(char *buf, int size, uint16_t timeout)
{
    char ch;
    int ind=0;

//    DPRINTLN(F("gets:"));

    while (readTimeout(&ch, timeout)) {
        if (ch == '\r') {
            readTimeout(&ch, timeout);
            if (ch == '\n') {
                if (buf) {
                    buf[ind] = 0;
                }
                return ind;
            }
            if (buf) {
                if (ind < (size-2)) {
                    buf[ind++] = '\r';
                    buf[ind++] = ch;
                } else if (ind < (size - 1)) {
                    buf[ind++] = '\r';
                }
            }
        }
        /* Truncate to buffer size */
        if ((ind < (size-1)) && buf) {
            buf[ind++] = ch;
        }
    }

    if (buf) {
        buf[ind] = 0;
    }
    return 0;
}


const char res_AOK[] PROGMEM = "AOK\r\n";
const char res_ERR[] PROGMEM = "ERR: ";
/* Get the result from a set operation
 * Should be AOK or ERR
 */
bool getres(char *buf, int size)
{
    const char *setResult[] = {
        PSTR("ERR: "),
        PSTR("AOK\r\n")
    };
    int8_t res;

//    DPRINTLN(F("getres"));

    res = multiMatch_P(setResult, 2,WIFLY_DEFAULT_TIMEOUT);

    if (res == 1) {
        return true;
    } else if (res == 0) {
        gets(buf, size,WIFLY_DEFAULT_TIMEOUT);
//        debug.print(F("ERR: "));
//        debug.println(buf);
    } else {
        /* timeout */
//        DPRINTLN(F("timeout"));
        strncpy_P(buf, PSTR("<timeout>"), size);
    }
    return false;
}

/* Finished with command */
bool finishCommand(void)
{
    if (--exitCommand == 0) {
        return exitCommandMode();
    }
    return true;
}


/** Take the WiFly out of command mode */
bool exitCommandMode(void)
{
    if (!inCommandMode) {
        return true;
    }

    send_P(PSTR("exit\r"));

    if (match_P(PSTR("EXIT\r\n"), WIFLY_DEFAULT_TIMEOUT)) {
        inCommandMode = false;
        return true;
    } else {
//        debug.println(F("Failed to exit\n\r"));
        return false;
    }
}


///* Get WiFly connection status */
//uint16_t getConnection(void)
//{
//    char buf[16];
//    uint16_t res;
//    int len;
//
//    if (!startCommand()) {
////        debug.println(F("getCon: failed to start"));
//        return 0;
//    }
//    //dbgBegin(256);
//
//    DPRINT(F("getCon\n\r"));
//    DPRINT(F("show c\n\r"));
//    send_P(PSTR("show c\r"));
//    len = gets(buf, sizeof(buf),WIFLY_DEFAULT_TIMEOUT);
//
//    if (checkPrompt(buf)) {
//        /* Got prompt first */
//        len = gets(buf, sizeof(buf),WIFLY_DEFAULT_TIMEOUT);
//    } else {
//        getPrompt(0);
//    }
//
//    if (len <= 4) {
//        res = (uint16_t)atoh(buf);
//    } else {
//        res = (uint16_t)atoh(&buf[len-4]);
//    }
//
//    status.tcp = (res >> WIFLY_STATUS_TCP_OFFSET) & WIFLY_STATUS_TCP_MASK;
//    status.assoc = (res >> WIFLY_STATUS_ASSOC_OFFSET) & WIFLY_STATUS_ASSOC_MASK;
//    status.authen = (res >> WIFLY_STATUS_AUTHEN_OFFSET) & WIFLY_STATUS_AUTHEN_MASK;
//    status.dnsServer = (res >> WIFLY_STATUS_DNS_SERVER_OFFSET) & WIFLY_STATUS_DNS_SERVER_MASK;
//    status.dnsFound = (res >> WIFLY_STATUS_DNS_FOUND_OFFSET) & WIFLY_STATUS_DNS_FOUND_MASK;
//    status.channel = (res >> WIFLY_STATUS_CHAN_OFFSET) & WIFLY_STATUS_CHAN_MASK;
//
//    finishCommand();
//
//    if (status.tcp == WIFLY_TCP_CONNECTED) {
//        connected = true;
////        if (debugOn) debug.println(F("getCon: TCP connected"));
//    } else {
//        connected = false;
////        if (debugOn) debug.println(F("getCon: TCP disconnected"));
//    }
//
//    return res;
//}

/** See if the prompt is somewhere in the string */
bool checkPrompt(const char *str)
{
    if (strstr(str, prompt) != NULL) {
        return true;
    } else {
        return false;
    }
}



/**
 * Read characters from the WiFly and match them against the set of
 * progmem strings. Ignore any leading characters that don't match. Keep
 * reading, discarding the input, until one of the strings is matched
 * or until no characters are received for the timeout duration.<br>
 * Example: res = multiMatch_P(500, 3, F("first="), F("second="), F("closed"));<br>
 * Will return 0 if "first=" is matched, 1 if "second=" is matched, 2 if "closed" is
 * matched, or -1 if nothing is matched and no data is received for 500 milliseconds.
 * @param timeout - fail if no data received for this period (in milliseconds).
 * @param count - the number of strings in the str array
 * @param ... A list of count progmem strings to match
 * @returns the index of the matching string
 * @retval -1 - no match found, timeout reached
 */
int multiMatch(uint16_t timeout, uint8_t count, ...)
{
    const char *str[20];
    int ind;
    va_list ap;
    va_start(ap, count);

    if (count > 20) {
        count = 20;
    }

    for (ind=0; ind<count; ind++) {
        str[ind] = va_arg(ap, const char *);
    }
    va_end(ap);

    return multiMatch_P(str, count, timeout);
}

/**
 * Read characters from the WiFly and match them against the set of
 * progmem strings. Ignore any leading characters that don't match. Keep
 * reading, discarding the input, until one of the strings is matched
 * or until no characters are received for the timeout duration.
 * @param str - the array of progmem strings to match
 * @param count - the number of strings in the str array
 * @param timeout - fail if no data received for this period (in milliseconds).
 * @returns the index of the matching string
 * @retval -1 - no match found, timeout reached
 */
int8_t multiMatch_P(const char *str[], uint8_t count, uint16_t timeout)
{
    struct {
        bool active;
        const char *str;
    } match[count];
    char ch, ch_P;
    uint8_t ind;

    for (ind=0; ind<count; ind++) {
        match[ind].active = false;
        match[ind].str = str[ind];
//        if (debugOn) {
//            debug.print(F("multiMatch_P: "));
//            debug.print(ind);
//            debug.print(' ');
//            debug.println((const __FlashStringHelper *)str[ind]);
//        }
    }

    while (readTimeout(&ch,timeout)) {
        for (ind=0; ind<count; ind++) {
            ch_P = pgm_read_byte(match[ind].str);
            if (ch == ch_P) {
                match[ind].str++;
            } else {
                /* Restart match */
                match[ind].str = str[ind];
                /* unmatched character might be the start of the string */
                if (ch == pgm_read_byte(match[ind].str)) {
                    match[ind].str++;
                }
            }

            if (pgm_read_byte(match[ind].str) == '\0') {
                /* Got a match */
//                DPRINTLN(F("multiMatch_P: true"));
                return ind;
            }
        }
    }

//    DPRINTLN(F("multiMatch_P: failed"));
    return -1;
}

/**
 * Read characters from the WiFly and match them against the
 * flash string. Ignore any leading characters that don't match. Keep
 * reading, discarding the input, until the string is matched
 * or until no characters are received for the timeout duration.
 * @param str The string to match (in flash memory)
 * @timeout fail if no data received for this period (in milliseconds).
 * @retval true - a match was found
 * @retval false - no match found, timeout reached
 */
bool match(const char *str, uint16_t timeout)
{
    return match_P((const char *)str, timeout);
}


//int8_t getDHCPMode(void)
//{
//    char buf[16];
//    int8_t mode;
//
//    if (!getopt(WIFLY_GET_DHCP, buf, sizeof(buf))) {
//        return -1;
//    }
//
//    if (strncmp_P(buf, PSTR("OFF"), 3) == 0) {
//        mode = 0;
//    } else if (strncmp_P(buf, PSTR("ON"), 2) == 0) {
//        mode = 1;
//    } else if (strncmp_P(buf, PSTR("AUTOIP"), 6) == 0) {
//        mode = 2;
//    } else if (strncmp_P(buf, PSTR("CACHE"), 5) == 0) {
//        mode = 3;
//    } else if (strncmp_P(buf, PSTR("SERVER"), 6) == 0) {
//        mode = 4;
//    } else {
//        mode = -1;      // unknown
//    }
//
//    return mode;
//}

//char getSpaceReplace(void)
//{
//    return (char)getopt_Asc(WIFLY_GET_REPLACE, HEX);
//}
//
//
//
/* Get the value of an option */
char *getopt(int opt, char *buf, int size)
{
    if (startCommand()) {
        send_P(requests[opt].req);

        if (match_P(requests[opt].resp, 500)) {
            gets(buf, size,WIFLY_DEFAULT_TIMEOUT);
            getPrompt(0);
            finishCommand();
            return buf;
        }

        finishCommand();
    }
    return (char *)"<error>";
}
//
//uint32_t getopt_Asc(int opt, uint8_t base)
//{
//    char buf[11];
//
//    if (!getopt(opt, buf, sizeof(buf))) {
//        return 0;
//    }
//
//    if (base == DEC) {
//        return atou(buf);
//    } else {
//        return atoh(buf);
//    }
//}


bool disableDHCP()
{
    return setDHCP(0);
}

bool enableDHCP()
{
    return setDHCP(1);
}

bool setSSID(const char *buf)
{
    return setopt_RM(PSTR("set wlan ssid"), (char *)buf, (char *)NULL, true);
}

bool setDHCP(const uint8_t mode)
{
    char buf[2];

    if (mode > 9) {
        return false;
    }

    buf[0] = '0' + mode;
    buf[1] = 0;

    return setopt_RM(PSTR("set ip dhcp"),(char *)buf, (char *)NULL, false);
//    return setopt_RM (PSTR("set ip dhcp"),buf,NULL, false);
}




/** Set the WiFi channel.
 * @param channel the wifi channel from 0 to 13. 0 means auto channel scan.
 * @returns true if successful, else false.
 */
bool setChannel(uint8_t channel)
{
    if (channel > 13) {
        channel = 13;
    }
    return setopt_p(PSTR("set wlan chan"), channel,DEC);
}

/** Set WEP key */
bool setKey(const char *buf)
{
    bool res;

    if ((buf[1] == 'x') || (buf[1] == 'X')) {
        /* Skip over the 0x leader */
        buf += 2;
    }

    res = setopt_RM(PSTR("set wlan key"), buf,NULL,false);

    hide();     /* hide the key */
    return res;
}


/**
 * Set WPA passphrase.
 * Spaces are automatically replaced with the current space substitution
 * character ('$' is the default).
 * @Note: If your passphrase contains a '$' then use setSpaceReplace() to
 *        change the replacement character to something you're not using.
 */
bool setPassphrase(const char *buf)
{
    bool res;
    res = setopt_RM(PSTR("set wlan phrase"), buf, NULL, true);

    hide();     /* hide the key */
    return res;
}

/** Hide passphrase and key */
bool hide(void)
{
    return setopt_RM(PSTR("set wlan hide 1"), (char *)NULL,NULL,true);
}

/** join a wireless network */
//bool join(const char *ssid, uint16_t timeout)
//{
//    int8_t res;
//    const char *joinResult[] = {
//        PSTR("FAILED"),
//        PSTR("Associated!")
//    };
//
//    if (!startCommand()) {
//        return false;
//    }
//
//    send_P(PSTR("join "));
//    if (ssid != NULL) {
//        sends(ssid);
//    }
//    send_P(PSTR("\r"));
//
//    res = multiMatch_P(joinResult,2,timeout);
//    flushRx(100);
//    if (res == 1) {
//        status.assoc = 1;
//        if (dhcp) {
//            // need some time to complete DHCP request
//            match_P(PSTR("GW="), 15000);
//            flushRx(100);
//        }
//        gets(NULL,0,WIFLY_DEFAULT_TIMEOUT);
//        finishCommand();
//        return true;
//    }
//
//    finishCommand();
//    return false;
//}


/** Get Version */
char * getVersion(char *buf, int size)
{
  return getopt(WIFLY_GET_VERSION, buf, size);
}


bool setJoin(uint8_t join)
{
    return setopt(PSTR("set wlan join"), join,DEC);
}
//
///**
// * Create an Adhoc WiFi network.
// * The WiFly is assigned IP address 169.254.1.1.
// * @param ssid the SSID to use for the network
// * @param channel the WiFi channel to use; 1 to 13.
// * @retval true - successfully create Ad Hoc network
// * @retval false - failed
// * @note the WiFly is rebooted as the final step of this command.
// */
uint8_t createAdhocNetwork(const char *ssid, uint8_t channel)
{

     Result=0;

    if (!startCommand()) Result = 1;
    if (Result==0) if(!setDHCP(WIFLY_DHCP_MODE_OFF)) Result = 2;
    if (Result==0) if(!setIP(Config.Wlan_IP)) Result = 3;
    if (Result==0) if(!setNetmask(Config.Wlan_Netmask)) Result = 4;
    if (Result==0) if(!setJoin(WIFLY_WLAN_JOIN_ADHOC)) Result = 5;
    if (Result==0) if(!setSSID(ssid)) Result = 6;
    if (Result==0) if(!setChannel(channel)) Result = 7;
    if (!save()) Result = 8;
//    _delay_ms(500);
    if (!finishCommand()) Result = 9;
    reboot();
    return Result;
}


/**
 * Set the WiFly's IP address.
 * @param buf IPv4 address in a null-terminated string.
 *            e.g "192.168.1.10"
 * @retval true address set
 * @retval false failed to set address
 */
bool setIP(const char *buf)
{
    return setopt_RM(PSTR("set ip address"),(char *) buf,NULL,false);
}

/** Reboots the WiFly.
* @note Depending on the shield, this may also reboot the Arduino.
*/
bool reboot(void)
{
    if (!startCommand()) {
return false;
    }
    send_P(PSTR("reboot\r"));
    if (!match_P(PSTR("*Reboot*"), WIFLY_DEFAULT_TIMEOUT)) {
finishCommand();
return false;
    }

    _delay_ms(5000);
    inCommandMode = false;
    exitCommand = 0;
//    wl_init();
    return true;

}


bool setHostIP(const char *buf)
{
    return setopt_RM(PSTR("set ip host"), buf,NULL,false);
}

bool setNetmask(const char *buf)
{
    return setopt_RM(PSTR("set ip netmask"), buf,NULL,false);
}

bool setGateway(const char *buf)
{
    return setopt_RM(PSTR("set ip gateway"), buf,NULL,false);
}

bool setBaud(uint8_t baud)
{
    char buf[16];
//    simple_utoa(baud, 10, buf, sizeof(buf));
//    DPRINT(F("set baud ")); DPRINT(buf); DPRINT("\n\r");





  switch (baud) {

      case Baud_2400   : strcpy_P( buf, PSTR("2400")); break;
      case Baud_4800   : strcpy_P( buf, PSTR("4800")); break;
      case Baud_9600   : strcpy_P( buf, PSTR("9600")); break;
      case Baud_19200  : strcpy_P( buf, PSTR("19200")); break;
      case Baud_38400  : strcpy_P( buf, PSTR("38400")); break;
      case Baud_57600  : strcpy_P( buf, PSTR("57600")); break;
      case Baud_115200 : strcpy_P( buf, PSTR("115200")); break;
      break;


                    }
    /* Go into command mode, since "set uart instant" will exit command mode */
    startCommand();
    if (setopt_RM(PSTR("set uart baud"), buf,NULL,false)) {
        //serial->begin(baud);          // Sketch will need to do this
        setopt_RM(PSTR("set uart instant"), buf,NULL,false);
        SetBaudUart1(Config.PKT_Baudrate);
        return true;


    }
    SetBaudUart1(Config.PKT_Baudrate);
    return true;
}


bool find_WiFly(void)  // search Wifly with all Baudrates

{
  uint8_t Result=false;


  lcd_cls ();
  lcd_printp_at( 0, 1, PSTR("search WiFly"), MNORMAL);
  Old_Baudrate = Config.PKT_Baudrate;
  if(enterCommandMode()) Result = true;


      if (!Result)
      {
        SetBaudUart1(Baud_9600);
        lcd_printp_at( 0, 2, PSTR("search 9600 Baud"), MNORMAL);
        if(enterCommandMode()) Result = true;}


      if (!Result)
        {
           SetBaudUart1(Baud_115200);
           lcd_printp_at( 0, 2, PSTR("search 115200 Baud"), MNORMAL);
           if(enterCommandMode()) Result = true;}

      if (!Result)
       {
         SetBaudUart1(Baud_38400 );
              lcd_printp_at( 0, 2, PSTR("search 38400 Baud"), MNORMAL);
         if(enterCommandMode()) Result = true;}

     if (!Result)
       {
     SetBaudUart1(Baud_19200);
         lcd_printp_at( 0, 2, PSTR("search 19200 Baud"), MNORMAL);
     if(enterCommandMode()) Result = true;}

     if (!Result)
       {
     SetBaudUart1( Baud_4800);
     lcd_printp_at( 0, 2, PSTR("search 4800 Baud"), MNORMAL);
     if(enterCommandMode()) Result = true;}

     if (!Result)
       {
     SetBaudUart1(Baud_2400);
     lcd_printp_at( 0, 2, PSTR("search 2400 Baud"), MNORMAL);
     if(enterCommandMode()) Result = true;}

     if (!Result)

    {
      lcd_printp_at( 0, 2, PSTR("WiFly FindError"), MNORMAL);
      lcd_printp_at( 0, 3, PSTR("Use config with PC"), MNORMAL);
      _delay_ms(1000);
      Result = false;
    }

  return Result;

}


#endif

